package com.good.ca.protocol.authentication;

import com.good.common.GDEntLogger;
import org.apache.http.client.HttpClient;
import org.apache.http.conn.ssl.AllowAllHostnameVerifier;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.conn.ssl.TrustStrategy;

import javax.net.ssl.KeyManager;
import javax.net.ssl.TrustManager;
import java.security.*;
import java.security.cert.X509Certificate;

/**
 * Class that returns {@link HttpClient} that trusts to all the incoming connection
 */
public class AllowAllSSLConnection extends AbstractSSLConnection {

    private static GDEntLogger logger = GDEntLogger.getInstance(AllowAllSSLConnection.class);

    protected AllowAllSSLConnection(SSLConnectionDTO sslConnectionDTO) throws HttpAuthenticationException {
        super(sslConnectionDTO);
        logger.debug("create AllowAllSSLConnection");
    }

    @Override
    protected TrustManager[] getTrustManagers(KeyStore trustStore, X509Certificate cert) throws KeyStoreException, NoSuchAlgorithmException {
        return AuthenticationUtils.getTrustManagersFromKeyStore(trustStore);
    }

    @Override
    protected SSLSocketFactory getSSLSocketFactory(KeyStore keyStore, KeyStore trustStore, KeyManager[] km, TrustManager[] tm)
            throws NoSuchAlgorithmException, KeyManagementException, UnrecoverableKeyException, KeyStoreException {
        TrustStrategy trustStrategy = (chain, authType) -> true;
        return new SSLSocketFactory("TLS", keyStore, decryptPassword,
                trustStore, null, trustStrategy,
                new AllowAllHostnameVerifier());
    }

    @Override
    protected KeyStore getTrustStore() throws HttpAuthenticationException, KeyStoreException, NoSuchAlgorithmException {
        //all connections are trusted by the trust strategy so we do not need to retrieve GC trustStore
        return KeyStore.getInstance("jks");
    }
}
