package com.good.ca.protocol.authentication;

import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.util.Objects;

/**
 * SSL connection authentication utils. Contains methods that will be used for SSL connection establishment.
 */

public class AuthenticationUtils {

    public static KeyManager[] getKeyManagersFromKeyStore(KeyStore keyStore, String password)
            throws NoSuchAlgorithmException, UnrecoverableKeyException, KeyStoreException {
        if (Objects.isNull(keyStore) || Objects.isNull(password)) {
            return null;
        }
        final KeyManagerFactory kmfactory = KeyManagerFactory.getInstance(KeyManagerFactory.getDefaultAlgorithm());
        kmfactory.init(keyStore, password.toCharArray());
        return kmfactory.getKeyManagers();
    }

    public static TrustManager[] getTrustManagersFromKeyStore(KeyStore trustStore)
            throws KeyStoreException, NoSuchAlgorithmException {
        TrustManagerFactory tmf = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
        tmf.init(trustStore);
        return tmf.getTrustManagers();
    }
}
