package com.good.ca.protocol.authentication;

import com.good.common.GDEntLogger;
import org.apache.commons.lang.StringUtils;
import org.apache.http.client.HttpClient;
import org.apache.http.conn.ssl.BrowserCompatHostnameVerifier;
import org.apache.http.conn.ssl.SSLSocketFactory;

import javax.net.ssl.KeyManager;
import javax.net.ssl.TrustManager;
import java.security.*;
import java.security.cert.X509Certificate;

/**
 * Class that returns {@link HttpClient}. It validates incoming server certificate upon SSL handshake
 * with uploaded CA certificate and GC TrustStore
 */

public class CACertificateSSLConnection extends AbstractSSLConnection {
    private static GDEntLogger logger = GDEntLogger.getInstance(CACertificateSSLConnection.class);

    protected CACertificateSSLConnection(SSLConnectionDTO sslConnectionDTO) throws HttpAuthenticationException {
        super(sslConnectionDTO);
        logger.debug("create CACertificateSSLConnection");
        if (StringUtils.isEmpty(sslConnectionDTO.getCaCertificate())) {
            throw new HttpAuthenticationException("CA Certificate cannot be null");
        }
        this.certificate = sslConnectionDTO.getCaCertificate();
    }

    @Override
    protected TrustManager[] getTrustManagers(KeyStore trustStore, X509Certificate cert) throws KeyStoreException, NoSuchAlgorithmException {
        return AuthenticationUtils.getTrustManagersFromKeyStore(trustStore);
    }

    @Override
    protected SSLSocketFactory getSSLSocketFactory(KeyStore keyStore, KeyStore trustStore, KeyManager[] km, TrustManager[] tm)
            throws NoSuchAlgorithmException, KeyManagementException {
        return this.getSSLSocketFactory(km, tm, new BrowserCompatHostnameVerifier());
    }

    @Override
    protected KeyStore getTrustStore() throws HttpAuthenticationException, KeyStoreException, NoSuchAlgorithmException {
        return this.getTrustStoreFromGC();
    }
}
