package com.good.ca.protocol.authentication;

/**
 * DTO with incoming parameters for {@link SSLConnection}
 */

public class SSLConnectionDTO {
    /**
     * Uploaded client certificate
     */
    private final String store;

    /**
     * password for client certificate
     */
    private final String decryptPassword;

    /**
     * uploaded server SSL certificate for validation of incoming server certificate upon SSL handshake
     */
    private final String serverSslCertificate;

    /**
     * uploaded CA certificate for validation of incoming server certificate upon SSL handshake
     */
    private final String caCertificate;

    private final PkiConnectionTypeEnum pkiConnectionType;

    private SSLConnectionDTO(Builder builder) {
        this.store = builder.store;
        this.decryptPassword = builder.decryptPassword;
        this.serverSslCertificate = builder.serverSslCertificate;
        this.caCertificate = builder.caCertificate;
        this.pkiConnectionType = builder.pkiConnectionType;
    }

    public String getCaCertificate() {
        return caCertificate;
    }

    public String getStore() {
        return store;
    }

    public String getDecryptPassword() {
        return decryptPassword;
    }

    public String getServerSslCertificate() {
        return serverSslCertificate;
    }

    public PkiConnectionTypeEnum getPkiConnectionType() {
        return pkiConnectionType;
    }

    public static class Builder {
        private String store;
        private String decryptPassword;
        private String serverSslCertificate;
        private String caCertificate;
        private PkiConnectionTypeEnum pkiConnectionType;

        public Builder() {
            this.pkiConnectionType = PkiConnectionTypeEnum.CA_FROM_TRUST_STORE;
        }

        public Builder(String store, String decryptPassword) {
            this();
            this.store = store;
            this.decryptPassword = decryptPassword;
        }

        public Builder serverSslCertificate(String serverSslCertificate) {
            this.serverSslCertificate = serverSslCertificate;
            if (serverSslCertificate != null) {
                this.pkiConnectionType = PkiConnectionTypeEnum.SERVER_SSL_CERTIFICATE;
            }
            return this;
        }

        public Builder caCertificate(String caCertificate) {
            this.caCertificate = caCertificate;
            if (this.caCertificate != null) {
                this.pkiConnectionType = PkiConnectionTypeEnum.CA_CERTIFICATE;
            }
            return this;
        }

        public SSLConnectionDTO build() {
            return new SSLConnectionDTO(this);
        }
    }

    public enum PkiConnectionTypeEnum {
        CA_FROM_TRUST_STORE, CA_CERTIFICATE, SERVER_SSL_CERTIFICATE;
    }
}
