package com.good.ca.protocol.authentication;

import com.good.ca.protocol.ConnectionStatusCode;
import com.good.common.GDEntLogger;

import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

/**
 * Factory that provides different implementation of {@link AbstractSSLConnection} based on incoming parameters
 */
public class SSLConnectionFactory {

    private static GDEntLogger logger = GDEntLogger.getInstance(SSLConnectionFactory.class);
    private static Map<SSLConnectionDTO.PkiConnectionTypeEnum, Class<? extends AbstractSSLConnection>> sslConnectionMap = new HashMap<>();
    static {
        sslConnectionMap.put(SSLConnectionDTO.PkiConnectionTypeEnum.CA_CERTIFICATE, CACertificateSSLConnection.class);
        sslConnectionMap.put(SSLConnectionDTO.PkiConnectionTypeEnum.SERVER_SSL_CERTIFICATE, ServerCertificateSSLConnection.class);
        sslConnectionMap.put(SSLConnectionDTO.PkiConnectionTypeEnum.CA_FROM_TRUST_STORE, AllowAllSSLConnection.class);
    }

    public static AbstractSSLConnection getSSLConnection(SSLConnectionDTO sslConnectionDTO) throws HttpAuthenticationException {
        logger.debug("pkiConnection type = " + sslConnectionDTO.getPkiConnectionType());
        try {
            return sslConnectionMap.get(sslConnectionDTO.getPkiConnectionType())
                    .getDeclaredConstructor(SSLConnectionDTO.class).newInstance(sslConnectionDTO);
        } catch (InvocationTargetException | NoSuchMethodException | InstantiationException | IllegalAccessException e) {
            logger.error("Internal Error happened upon SSL connection initialization. ", e);
            throw new HttpAuthenticationException("Internal Error happened upon SSL connection initialization. ",
                    e, ConnectionStatusCode.SERVICE_EXCEPTION);
        }
    }
}
