package com.good.ca.protocol.authentication;

import com.good.common.GDEntLogger;
import org.apache.commons.lang.StringUtils;
import org.apache.http.conn.ssl.*;
import org.apache.http.client.HttpClient;

import javax.net.ssl.KeyManager;
import javax.net.ssl.TrustManager;
import java.security.*;
import java.security.cert.X509Certificate;

/**
 * Class that returns {@link HttpClient}. It validates incoming server certificate upon SSL handshake
 * with uploaded server SSL certificate and GC TrustStore
 */

public class ServerCertificateSSLConnection extends AbstractSSLConnection {

    private static GDEntLogger logger = GDEntLogger.getInstance(ServerCertificateSSLConnection.class);

    protected ServerCertificateSSLConnection(SSLConnectionDTO sslConnectionDTO) throws HttpAuthenticationException {
        super(sslConnectionDTO);
        logger.debug("create ServerCertificateSSLConnection");
        if (StringUtils.isEmpty(sslConnectionDTO.getServerSslCertificate())) {
            throw new HttpAuthenticationException("ServerSslCertificate cannot be null");
        }
        this.certificate = sslConnectionDTO.getServerSslCertificate();
    }

    @Override
    protected TrustManager[] getTrustManagers(KeyStore trustStore, X509Certificate cert) throws KeyStoreException, NoSuchAlgorithmException {
        return new TrustManager[]{
                new ServerSSLTrustManager(trustStore, cert)
        };
    }

    @Override
    protected SSLSocketFactory getSSLSocketFactory(KeyStore keyStore, KeyStore trustStore, KeyManager[] km, TrustManager[] tm)
            throws NoSuchAlgorithmException, KeyManagementException {
        return this.getSSLSocketFactory(km, tm, new AllowAllHostnameVerifier());
    }

    @Override
    protected KeyStore getTrustStore() throws HttpAuthenticationException, KeyStoreException, NoSuchAlgorithmException {
        return this.getTrustStoreFromGC();
    }
}
