package com.good.ca.protocol.authentication;

import com.good.common.GDEntLogger;

import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.X509TrustManager;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

/**
 * TrustManager that upon execution of checkServerTrusted method validates
 * whether incoming certificate and certificate that was uploaded in GC have the same end date
 */

public class ServerSSLTrustManager implements X509TrustManager {
    private static final GDEntLogger logger = GDEntLogger.getInstance(ServerSSLTrustManager.class);
    private KeyStore trustStore;
    private X509TrustManager trustManager;
    private X509Certificate cert;

    public ServerSSLTrustManager(KeyStore ts, X509Certificate cert) throws KeyStoreException, NoSuchAlgorithmException {
        logger.debug("ServerSSLTrustManager");
        this.trustStore = ts;
        this.cert = cert;
        reloadTrustManager();
    }

    @Override
    public void checkClientTrusted(X509Certificate[] chain,
                                   String authType) throws CertificateException {
        logger.debug("checkClientTrusted");
        trustManager.checkClientTrusted(chain, authType);
    }

    @Override
    public void checkServerTrusted(X509Certificate[] chain,
                                   String authType) throws CertificateException {
        logger.debug("checkServerTrusted");
        if (this.cert == null || !this.cert.getNotAfter().equals(chain[0].getNotAfter())) {
            logger.debug("checkServerTrusted cert is null or dates are not equal");
            throw new CertificateException("Expiration dates of server SSL certificates are not equal");
        }
    }

    @Override
    public X509Certificate[] getAcceptedIssuers() {
        return trustManager.getAcceptedIssuers();
    }

    private void reloadTrustManager() throws NoSuchAlgorithmException, KeyStoreException {
        TrustManagerFactory tmf = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
        tmf.init(trustStore);

        TrustManager tms[] = tmf.getTrustManagers();
        for (TrustManager tm : tms) {
            if (tm instanceof X509TrustManager) {
                trustManager = (X509TrustManager) tm;
                return;
            }
        }
        throw new NoSuchAlgorithmException("No X509TrustManager in TrustManagerFactory");
    }
}