package com.good.ca.protocol.bean;

import com.google.gson.JsonObject;

import java.util.Objects;

public class RequestCAKeyPair extends RequestCertificateAuthority {

    private final String otp;
    private final String userEmail;
    private final String header;
    private final String signature;
    private final RequestType requestType;
    private final String deviceId;
    private final String deviceName;

    public RequestCAKeyPair(Builder builder) {
        this.setHttpType();
        this.otp = builder.otp;
        this.userEmail = builder.userEmail;
        this.header = builder.header;
        this.signature = builder.signature;
        this.requestType = builder.requestType;
        this.deviceId = builder.deviceId;
        this.deviceName = builder.deviceName;
    }

    public static class Builder {
        private final RequestType requestType;
        private String otp;
        private String userEmail;
        private String header;
        private String signature;
        private String deviceId;
        private String deviceName;

        public Builder(RequestType requestType) {
            this.requestType = requestType;
        }

        public Builder otp(String otp) {
            this.otp = otp;
            return this;
        }

        public Builder userEmail(String userEmail) {
            this.userEmail = userEmail;
            return this;
        }

        public Builder header(String header) {
            this.header = header;
            return this;
        }

        public Builder signature(String signature) {
            this.signature = signature;
            return this;
        }

        public Builder deviceId(String deviceId) {
            this.deviceId = deviceId;
            return this;
        }

        public Builder deviceName(String deviceName) {
            this.deviceName = deviceName;
            return this;
        }

        public RequestCAKeyPair build() {
            return new RequestCAKeyPair(this);
        }
    }

    public String getOtp() {
        return otp;
    }

    public String getUserEmail() {
        return userEmail;
    }

    public String getHeader() {
        return header;
    }

    public String getSignature() {
        return signature;
    }

    public RequestType getRequestType() {
        return requestType;
    }

    public String getDeviceId() {
        return deviceId;
    }

    public String getDeviceName() {
        return deviceName;
    }

    @Override
    public JsonObject getMessage() {
        JsonObject message = new JsonObject();
        message.addProperty("reqId", this.getRequestId());
        message.addProperty("mType", requestType.getType());
        message.addProperty("user", this.userEmail);
        message.addProperty("deviceId", deviceId);
        message.addProperty("deviceName", deviceName);

        if (requestType == RequestType.INITIAL_CERT) {
            message.addProperty("authToken", this.otp);
        }
        if (requestType == RequestType.RENEW_CERT) {
            message.addProperty("signature", this.getSignature());
            message.addProperty("header", this.getHeader());
        }
        return message;
    }

    @Override
    public void setHttpType() {
        this.httpType = HttpType.POST;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        RequestCAKeyPair that = (RequestCAKeyPair) o;
        return Objects.equals(otp, that.otp) &&
                Objects.equals(userEmail, that.userEmail) &&
                Objects.equals(header, that.header) &&
                Objects.equals(signature, that.signature) &&
                requestType == that.requestType &&
                Objects.equals(deviceId, that.deviceId) &&
                Objects.equals(deviceName, that.deviceName);
    }

    @Override
    public int hashCode() {
        return Objects.hash(otp, userEmail, header, signature, requestType, deviceId, deviceName);
    }
}
