package com.good.ca.protocol.bean;

import com.good.crypto.util.Base64String;
import com.good.util.validation.annotation.NotEmpty;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;

import java.util.List;
import java.util.Objects;

/**
 * Request for notifyCertificateReceived operation
 */
public class RequestNotifyCertificateReceived extends RequestCertificateAuthority {

	/**
	 * User email addr
	 */
	@NotEmpty
	private String user;
	/**
	 * Certificate received
	 */
	@NotEmpty
	private Base64String receivedCert;
	/**
	 * List of other certs present on the device ie. previous certs
	 */
	
	private List<Base64String> otherCerts; // This can be null
	/**
	 * GD device id. In some edge cases it is possible for GD container to change its deviceId.
	 */
	private String deviceId;
	/**
	 * Name of the device assigned by the user, which user can change at a later time.
	 */
	private String deviceName;
	
	public RequestNotifyCertificateReceived(String user,Base64String receivedCert,List<Base64String> otherCerts, String deviceId,String deviceName) {
		this.user = user;
		this.otherCerts = otherCerts;
		this.receivedCert = receivedCert;
		this.deviceId = deviceId;
		this.deviceName = deviceName;
		checkRequiredFields(this);
	}
	

	private RequestNotifyCertificateReceived(Builder builder) {
		this.user = builder.user;
		this.otherCerts = builder.otherCerts;
		this.receivedCert = builder.receivedCert;
		this.deviceId = builder.deviceId;
		this.deviceName = builder.deviceName;
		checkRequiredFields(this);
	}
	public static class Builder{
		private String user;
		private List<Base64String> otherCerts;
		private Base64String receivedCert;
		private String deviceId;
		private String deviceName;

		public Builder(String user) {
			this.user = user;
		}

		public Builder otherCerts(List<Base64String> otherCerts) {
			this.otherCerts = otherCerts;
			return this;
		}

		public Builder receivedCert(Base64String receivedCert) {
			this.receivedCert = receivedCert;
			return this;
		}

		public Builder deviceId(String deviceId) {
			this.deviceId = deviceId;
			return this;
		}

		public Builder deviceName(String deviceName) {
			this.deviceName = deviceName;
			return this;
		}

		public RequestNotifyCertificateReceived build() {
			return new RequestNotifyCertificateReceived(this);
		}
	}

	public String getUser() {
		return user;
	}

	public List<Base64String> getOtherCerts() {
		return otherCerts ;
	}

	public Base64String getReceivedCert() {
		return receivedCert;
	}

	public String getDeviceId() {
		return deviceId;
	}

	public String getDeviceName() {
		return deviceName;
	}

	@Override
	public String toString() {
		return "RequestNotifyCertificateReceived {" +
				"user='" + user + '\'' +
				", otherCerts=" + otherCerts +
				", receivedCert='" + receivedCert + '\'' +
				", deviceId='" + deviceId + '\'' +
				", deviceName='" + deviceName + '\'' +
				'}';
	}

	@Override
	public JsonObject getMessage() {
		JsonObject message = new JsonObject();
		message.addProperty("user", user);
		JsonArray otherCertsArray = new JsonArray();
		for (Base64String otherCert : otherCerts) {
			otherCertsArray.add(new JsonPrimitive(otherCert.toString()));
		}
		message.add("otherCerts", otherCertsArray);
		message.addProperty("receivedCert", receivedCert.toString());
		message.addProperty("deviceId", deviceId);
		message.addProperty("deviceName", deviceName);

		return message;
	}

	@Override
	public void setHttpType() {
		this.httpType = RequestCertificateAuthority.HttpType.POST;
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) return true;
		if (o == null || getClass() != o.getClass()) return false;
		RequestNotifyCertificateReceived that = (RequestNotifyCertificateReceived) o;
		return  Objects.equals(user, that.user) &&
				(otherCerts==null ? that.otherCerts==null : otherCerts.equals(that.otherCerts)) &&
				Objects.equals(receivedCert, that.receivedCert) &&
				Objects.equals(deviceId, that.deviceId) &&
				Objects.equals(deviceName, that.deviceName);
	}

	@Override
	public int hashCode() {
		return Objects.hash(user, otherCerts, receivedCert, deviceId, deviceName);
	}
}
