package com.good.ca.protocol.bean;

import com.good.util.validation.annotation.NotEmpty;
import com.good.crypto.util.Base64String;
import com.google.gson.JsonObject;

import java.util.Objects;

/**
 * signCertificate operation request
 *
 */
public class RequestSignCertificate extends RequestCertificateAuthority {
    /**
     * {"renewCert", “initialCert”}
     */
    @NotEmpty
    private final RequestType mType;
    /**
     * User email addr
     */
    @NotEmpty
    private final String user;
    /**
     * GC server sends this to the PKI Connector as received without doing any changes.
     */
    @NotEmpty
    private final Base64String cmsSigned;

    private RequestSignCertificate(Builder builder) {
        this.mType = builder.mType;
        this.user = builder.user;
        this.cmsSigned = builder.cmsSigned;
        checkRequiredFields(this);
    }

    public static class Builder {
        private RequestType mType;
        private String user;
        private Base64String cmsSigned;

        public Builder (RequestType mType) {
            this.mType = mType;
        }

        public Builder user (String user) {
            this.user = user;
            return this;
        }

        public Builder cmsSigned (Base64String cmsSigned) {
            this.cmsSigned = cmsSigned;
            return this;
        }

        public RequestSignCertificate build() {
            return new RequestSignCertificate(this);
        }
    }

    public RequestType getmType() {
        return mType;
    }

    public String getUser() {
        return user;
    }

    public Base64String getCmsSigned() {
        return cmsSigned;
    }

    @Override
    public String toString() {
        return "RequestSignCertificate{" +
                "mType=" + mType.getType() +
                ", user='" + user + '\'' +
                ", cmsSigned=" + cmsSigned +
                '}';
    }

    @Override
    public JsonObject getMessage() {
        JsonObject message = new JsonObject();
        message.addProperty("mType", mType.getType());
        message.addProperty("user", user);
        message.addProperty("cmsSigned", cmsSigned.toString());

        return message;
    }

    @Override
    public void setHttpType() {
        this.httpType = HttpType.POST;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        RequestSignCertificate that = (RequestSignCertificate) o;
        return Objects.equals(mType, that.mType) &&
                Objects.equals(user, that.user) &&
                Objects.equals(cmsSigned, that.cmsSigned);
    }

    @Override
    public int hashCode() {
        return Objects.hash(mType, user, cmsSigned);
    }
}
