package com.good.ca.protocol.bean;

import com.google.common.base.CaseFormat;
import org.json.JSONException;

import java.util.Objects;

public class ResponseCAKeyPair extends ResponseCertificateAuthority {

	public ResponseCAKeyPair(String json) throws JSONException {
		super(json);
	}

	public enum PayLoadType {
		PKCS12("pkcs12", "payload"),
		X509("x509List", "payloads");
		
		private String cmpType;
		private String payloadName;

		PayLoadType(String cmpType, String payloadName) {
			this.cmpType = cmpType;
			this.payloadName = payloadName;
		}

		public String getCmpType() {
			return cmpType;
		}

		public String getPayloadName() {
			return payloadName;
		}
	}

	public Object getPayLoad() throws JSONException {
		PayLoadType payLoadType = getPayLoadType();
		return getOptionalObject(payLoadType.getPayloadName());
	}

	public String getPassword() throws JSONException {
		return getOptionalString("password");
	}

	public String getStatus() throws JSONException {
		return getRequiredString("status");
	}

	public PayLoadType getPayLoadType() throws JSONException {
		String type = getRequiredString("payloadType");
		if(Objects.isNull(type)) {
			throw new JSONException("Mandatory field payloadType is absent");
		}
		return PayLoadType.valueOf(CaseFormat.LOWER_CAMEL.to(CaseFormat.UPPER_UNDERSCORE, type));
	}
	
	public String getFailureInfo() throws JSONException {
		return getOptionalString("failureInfo");
	}

    public String getReqId() throws JSONException {
        return getOptionalString("reqId");
    }

	@Override
	public String toString() {
		return "ResponseCAKeyPair{" +
				"payLoad=" + getJson() +
				'}';
	}
}
