package com.good.ca.protocol.bean;

import com.good.crypto.util.Base64String;
import org.json.JSONException;

/**
 * Response for getUserKeyPair2 operation
 */
public class ResponseCAKeyPair2 extends ResponseCertificateAuthority {
    /**
     * Status of operation(success/failure). Mandatory field.
     */
    private String status = null;
    /**
     * Information about failure
     */
    private String failureInfo = null;
    /**
     * Type of keystore
     */
    private String payloadType = null;
    /**
     * Base64 encoded p12
     */
    private Base64String payload = null;
    /**
     * request ID should be same as field in {@link RequestCAKeyPair2#reqId}. Is needed for client to separate
     * different responses.
     */
    private String reqId = null;
    /**
     * If pkcs12 was password encrypted and authToken was not used for encryption, decryption password may be returned here.
     * For Renew case password is needed
     */
    private String password = null;

    public ResponseCAKeyPair2(String json) throws JSONException {
        super(json);
        parseResponse();
    }

    public String getStatus() throws JSONException {
        return status;
    }

    public String getFailureInfo() throws JSONException {
        return failureInfo;
    }

    public String getPayloadType() throws JSONException {
        return payloadType;
    }

    public Base64String getPayload() throws JSONException {
        return payload;
    }

    public String getReqId() throws JSONException {
        return reqId;
    }

    public String getPassword() throws JSONException {
        return password;
    }

    @Override
    public String toString() {
        return "ResponseCAKeyPair2{" +
                "status=" + status +
                ", failureInfo='" + failureInfo + '\'' +
                ", payloadType='" + "payloadType" + '\'' +
                ", payload=" + payload +
                ", reqId='" + reqId + '\'' +
                ", password='" + "***" + '\'' +
                '}';
    }

    private void parseResponse() throws JSONException {
        status = getRequiredString("status");
        failureInfo = getOptionalString("failureInfo");
        payloadType = getOptionalString("payloadType");
        payload = new Base64String(getOptionalString("payload"));
        reqId = getRequiredString("reqId");
        password = getOptionalString("password");
    }
}
