package com.good.ca.protocol.bean;

import org.json.JSONArray;
import org.json.JSONException;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Response for notifyCertificateReceived operation
 */
public class ResponseNotifyCertificateReceived extends ResponseCertificateAuthority {
    /**
     * Status of operation(success/failure). Mandatory field.
     */
    private String status = null;
    /**
     * Information about failure
     */
    private String failureInfo = null;
    /**
     * Certificates that GC should delete
     */
    private List<String> removeCerts = new ArrayList<String>();

    public ResponseNotifyCertificateReceived(String json) throws JSONException {
        super(json);
        parseResponse();
    }

    public String getStatus() throws JSONException {
        return status;
    }

    public String getFailureInfo() throws JSONException {
        return failureInfo;
    }

    public List<String> getRemoveCerts() throws JSONException {
    	return Collections.unmodifiableList(removeCerts); //not sure if this is needed, but keeping the existing logic.. 
    }

    @Override
    public String toString() {
        return "ResponseNotifyCertificateReceived{" +
                "status=" + status +
                ", failureInfo='" + failureInfo + '\'' +
                ", removeCerts=" + (removeCerts) +
                '}';
    }

    private void parseResponse() throws JSONException {
        status = getRequiredString("status");
        failureInfo = getOptionalString("failureInfo");
        JSONArray jsonArray = getOptionalArray("removeCerts");
        if (jsonArray != null) {
            for (int i = 0; i < jsonArray.length(); i++) {
                removeCerts.add(jsonArray.get(i).toString());
            }
        }
    }
    public boolean isRetryable(){
    	if(status != null && "failure".equalsIgnoreCase(status) && failureInfo != null && "retry".equalsIgnoreCase(failureInfo))
    		return true;
    	
    	return false;
    				
    }
}
