package com.good.ca.protocol.bean;

import com.good.common.GDEntLogger;
import com.good.crypto.util.Base64String;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.Arrays;
import java.util.Objects;

/**
 * Response to signCertificate operation
 */
public class ResponseSignCertificate extends ResponseCertificateAuthority {

    private static final GDEntLogger LOGGER = GDEntLogger.getInstance(ResponseSignCertificate.class);

    /**
     * Status of operation(success/failure). Mandatory field.
     */
    private String status = null;
    /**
     * Information about failure
     */
    private String failureInfo = null;
    /**
     * Type of keystore
     */
    private String payloadType = null;
    /**
     * Base64 encoded payload. First certificate in the list must be user’s public cert.
     * Cert chain can be returned subsequently in the list.
     */
    private Base64String[] payloads = null;
    /**
     * request ID should be same as field in {@link RequestSignCertificate}. Is needed for client to separate
     * different responses.
     */
    private String reqId = null;

    public ResponseSignCertificate(String json) throws JSONException {
        super(json);
        parseResponse();
    }

    public String getStatus() throws JSONException {
        return status;
    }

    public String getFailureInfo() throws JSONException {
        return failureInfo;
    }

    public String getPayloadType() throws JSONException {
        return payloadType;
    }

    public Base64String[] getPayloads() throws JSONException {
        return payloads;
    }

    public String getReqId() throws JSONException {
        return reqId;
    }

    @Override
    public String toString() {
        return "ResponseSignCertificate{" +
                "status=" + status +
                ", failureInfo='" + failureInfo + '\'' +
                ", payloadType='" + payloadType + '\'' +
                ", payloads=" + Arrays.toString(payloads) +
                ", reqId='" + reqId + '\'' +
                '}';
    }

    private void parseResponse() throws JSONException {
        status = getRequiredString("status");
        failureInfo = getOptionalString("failureInfo");
        payloadType = getOptionalString("payloadType");
        payloads = new Base64String[1];
        final JSONObject jsonObject = getJSONObject();
        final JSONArray payloadsArray = jsonObject.optJSONArray("payloads");
        if(Objects.nonNull(payloadsArray) && !payloadsArray.isNull(0)) {
            this.payloads[0] = new Base64String(payloadsArray.getString(0));
        }
        reqId = getRequiredString("reqId");
    }
}
