package com.good.ca.protocol.model;

import com.good.ca.protocol.bean.ResponseCAKeyPair;
import com.google.common.base.CaseFormat;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonPrimitive;
import com.google.gson.JsonSerializer;

import java.util.Objects;

/**
 * Model class fo testing, which contains mock response data for getKeyPair request to real end-point
 *
 * Created by omoniat on 17.03.2016.
 */
public class KeyPairResponseData implements JSONConvertible {
    
    private static final Gson JSON_SERIALIZER;
    static {
        JsonSerializer<ResponseCAKeyPair.PayLoadType> payloadSerializer = (val, type, context) -> {
            if(Objects.isNull(val)) {
                return null;
            }
            return new JsonPrimitive(CaseFormat.UPPER_UNDERSCORE.to(CaseFormat.LOWER_CAMEL, val.toString()));
        };
        JSON_SERIALIZER = new GsonBuilder()
                .registerTypeAdapter(ResponseCAKeyPair.PayLoadType.class, payloadSerializer)
                .create();
    }

    private String reqId;
    private String status;
    private String failureInfo;
    private String payload;
    private ResponseCAKeyPair.PayLoadType payloadType;
    private String password;

    public KeyPairResponseData(String reqId, String status, String failureInfo, String payLoad, ResponseCAKeyPair.PayLoadType payloadType, String password) {
        this.reqId = reqId;
        this.status = status;
        this.failureInfo = failureInfo;
        this.payload = payLoad;
        this.payloadType = payloadType;
        this.password = password;
    }

    public String toJSON(){
        return JSON_SERIALIZER.toJson(this);
    }

    public String getReqId() {
        return reqId;
    }

    public String getStatus() {
        return status;
    }

    public String getFailureInfo() {
        return failureInfo;
    }

    public String getPayload() {
        return payload;
    }

    public ResponseCAKeyPair.PayLoadType getPayLoadType() {
        return payloadType;
    }

    public String getPassword() {
        return password;
    }

}
